<?php

namespace Drupal\rac\Plugin\adva\AccessProvider;

use Drupal\adva\Plugin\adva\ReferenceAccessProvider;

use Drupal\Core\Entity\ContentEntityTypeInterface;
use Drupal\Core\Entity\EntityFieldManagerInterface;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityTypeBundleInfo;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Session\AccountInterface;

use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Role Access Provider for Advanced Access.
 *
 * Implements access control using reference fields with a target type of
 * user_role.
 *
 * View grants are given to each of the roles referenced from a entity. Update
 * grants are determined by the update permissions.
 *
 * @AccessProvider(
 *   id = "rac",
 *   label = @Translation("Role Access Control"),
 *   operations = {
 *     "view",
 *     "update",
 *     "delete",
 *   },
 * )
 */
class RoleAccessProvider extends ReferenceAccessProvider {

  /**
   * {@inheritdoc}
   */
  public static function getTargetType() {
    return "user_role";
  }

  /**
   * {@inheritdoc}
   */
  public function getAccessRecords(EntityInterface $entity) {
    $roles = $this->getReferencedItems($entity);
    if (count($roles)) {
      // If we're here, the node has roles associated with it which restrict
      // access to the node.
      $grants = [];
      $grant_delete = isset($this->configuration['grant_delete']) && $this->configuration['grant_delete'];

      foreach ($roles as $role) {
        $grant = [
          'realm' => "rac_" . $role,
          'gid' => 1,
          'grant_view' => 1,
          'grant_update' => 0,
          'grant_delete' => 0,
          'priority' => 0,
        ];
        $grant2 = [
          'realm' => "rac_" . $role,
          'gid' => 2,
          'grant_view' => 1,
          'grant_update' => 1,
          'grant_delete' => $grant_delete ? 1 : 0,
          'priority' => 0,
        ];

        if ($entity->getEntityType()->id() === "node") {
          if ($entity->isPublished()) {
            $grants[] = $grant;
            $grants[] = $grant2;
          } else if (_rac_update_unpublished()) {
            $grants[] = $grant2;
          }
        } else {
          $grants[] = $grant;
          $grants[] = $grant2;
        }
      }
      return $grants;
    }
  }

  /**
   * {@inheritdoc}
   */
  public function getAccessGrants($operation, AccountInterface $account) {
    $grants = [];

    $access = ($operation === 'view') ? 1 : 2;
    $role_operation = ($access === 1) ? 'view' : 'update';

    $userRoles = _rac_get_account_roles($role_operation, $account);
    foreach ($userRoles as $role) {
      $grants["rac_" . $role->id()][] = $access;
    }

    if ($access === 1) {
      /*
       * For view access, check if the user has edit access. This is required if
       * the content is unpublishes, since the view grant records will not be
       * stored, but update grants view.
       */
      $update_grants = $this->getAccessGrants("update", $account);
      if ($update_grants) {
        $grants = array_merge_recursive($grants, $update_grants);
      }
    }

    if (count($grants)) {
      return $grants;
    }
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildConfigForm($form, $form_state);

    $consumer = $this->getConsumer();

    $form['grant_delete'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Allow editors to delete content'),
      '#description' => $this->t('This will allow a user to delete any content he/she can edit.'),
      '#default_value' => $this->configuration['grant_delete'] ?: FALSE,
      '#parents' => [
        'consumers',
        $consumer->getPluginId(),
        'config',
        'providers',
        $this->getPluginId(),
        'grant_delete',
      ],
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigForm(array &$form, FormStateInterface $form_state) {
    $consumer = $this->getConsumer();
    $values = $form_state->getValues();
    $this->configuration['grant_delete'] = (bool) $values['consumers'][$consumer->getPluginId()]['config']['providers'][$this->getPluginId()]['grant_delete'];
  }

}
